searchData={"items":[{"type":"module","title":"PlugLocale.Browser","doc":"Puts locale into `assigns` storage for Web browser environment.\n\nThe most common way of specifying the desired locale is via the URL. In\ngeneral, there're three methods to do that:\n\n  1. via domain name - `https:// .example.com`, such as:\n     * `https://en.example.com/welcome`\n     * `https://zh.example.com/welcome`\n  2. via path - `https://example.com/ `, such as:\n     * `https://example.com/en/welcome`\n     * `https://example.com/zh/welcome`\n  3. via querystring - `https://example.com?locale= `, such as:\n     * `https://example.com/welcome?locale=en`\n     * `https://example.com/welcome?locale=zh`\n\nPersonally, I think method 2 is better, compared to the other two methods:\n\n  * method 1 is tedious for deployment.\n  * URLs generated by method 3 look very ugly and unprofessional.\n\nBecause of that, this plug will stick on method 2.","ref":"PlugLocale.Browser.html"},{"type":"module","title":"Usage - PlugLocale.Browser","doc":"First, we need to integrate this plug with other libraries, or this plug\nis useless. All you need is to construct a plug pipeline through\n`Plug.Builder`. For example:\n\n    defmodule DemoWeb.PlugBrowserLocalization do\n      use Plug.Builder\n    \n      plug PlugLocale.Browser,\n        default_locale: \"en\",\n        locales: [\"en\", \"zh\"],\n        route_identifier: :locale,\n        assign_key: :locale\n    \n      plug :put_locale\n    \n      def put_locale(conn, _opts) do\n        if locale = conn.assigns[:locale] do\n          # integrate with gettext\n          Gettext.put_locale(locale)\n        end\n    \n        conn\n      end\n    end\n\nThen, use it in router (following one is a Phoenix router, but `Plug.Router`\nis supported, too):\n\n    defmodule DemoWeb.Router do\n      use DemoWeb, :router\n    \n      pipeline :browser do\n        plug :accepts, [\"html\"]\n\n        # ...\n\n        plug DemoWeb.PlugBrowserLocalization\n\n        # ...\n      end\n    \n      scope \"/\", DemoWeb do\n        pipe_through :browser\n\n        get \"/\", PageController, :index\n        # ...\n      end\n\n      # Why using :locale?\n      # Because it is specified by `:route_identifier` option.\n      scope \"/:locale\", DemoWeb do\n        pipe_through :browser\n\n        get \"/\", PageController, :index\n        # ...\n      end\n    end","ref":"PlugLocale.Browser.html#module-usage"},{"type":"module","title":"Options - PlugLocale.Browser","doc":"* `:default_locale` - the default locale.\n  * `:locales` - all the supported locales.\n    Default to `[]`.\n  * `:detect_locale_from` - *the sources* and *the order of sources* for\n    detecting locale.\n    Available sources are `:query`, `:cookie`, `:referrer`, `:accept_language`.\n    Default to `[:cookie, :referrer, :accept_language]`.\n  * `:cast_locale_by` - the function for casting extracted or detected locales.\n    Default to `nil`.\n  * `:route_identifier` - the part for identifying locale in route.\n    Default to `:locale`.\n  * `:assign_key` - the key for putting value into `assigns` storage.\n    Default to `:locale`.\n  * `:query_key` - the key for getting locale from querystring.\n    Default to `\"locale\"`.\n  * `:cookie_key` - the key for getting locale from cookie.\n    Default to `\"locale\"`.\n\n#","ref":"PlugLocale.Browser.html#module-options"},{"type":"module","title":"about `:cast_locale_by` option - PlugLocale.Browser","doc":"By default, the value is `nil`, which means doing nothing. But, in practice,\nyou will need to use something meaningful.\n\nA possible implementation:\n\n    defmodule DemoWeb.I18n do\n      def cast_locale(locale) do\n        case locale do\n          # explicit matching on supported locales\n          locale when locale in [\"en\", \"zh\"] ->\n            locale\n\n          # fuzzy matching on en locale\n          \"en-\" <> _ ->\n            \"en\"\n\n          # fuzzy matching on zh locale\n          \"zh-\" <> _ ->\n            \"zh\"\n\n          # fallback for unsupported locales\n          _ ->\n            \"en\"\n        end\n      end\n    end\n\nThen, use above implementation for plug:\n\n    plug `PlugLocale.Browser`,\n      default_locale: \"en\",\n      locales: [\"en\", \"zh\"],\n      cast_locale_by: &DemoWeb.I18n.cast_locale/1,\n      # ...","ref":"PlugLocale.Browser.html#module-about-cast_locale_by-option"},{"type":"module","title":"Helper functions - PlugLocale.Browser","doc":"`PlugLocale.Browser` also provides some helper functions, which will be useful\nwhen implementing UI components:\n\n  * `build_locale_path/2`\n  * `put_locale_resp_cookie/2` / `put_locale_resp_cookie/3`\n\nCheck out their docs for more details.\n\n#","ref":"PlugLocale.Browser.html#module-helper-functions"},{"type":"module","title":"an example - a simple locale switcher using `build_locale_path/2` - PlugLocale.Browser","doc":"```heex\n \n   \n     \n      English\n     \n   \n   \n     \n      中文\n     \n   \n \n```","ref":"PlugLocale.Browser.html#module-an-example-a-simple-locale-switcher-using-build_locale_path-2"},{"type":"module","title":"How it works? - PlugLocale.Browser","doc":"This plug will try to:\n\n  1. extract locale from URL, and check if the locale is supported:\n     - If it succeeds, put locale into `assigns` storage.\n     - If it fails, jump to step 2.\n  2. detect locale from Web browser environment, then redirect to\n     the path corresponding to detected locale.\n\n#","ref":"PlugLocale.Browser.html#module-how-it-works"},{"type":"module","title":"Extract locale from URL - PlugLocale.Browser","doc":"For example, the locale extracted from `https://example.com/en/welcome`\nis `en`.\n\n#","ref":"PlugLocale.Browser.html#module-extract-locale-from-url"},{"type":"module","title":"Detect locale from Web browser environment - PlugLocale.Browser","doc":"By default, local is detected from multiple sources:\n\n  * query (whose key is specified by `:query_key` option)\n  * cookie (whose key is specified by `:cookie_key` option)\n  * HTTP request header - `referer`\n  * HTTP request header - `accept-language`\n\nIf all detections fail, fallback to default locale.","ref":"PlugLocale.Browser.html#module-detect-locale-from-web-browser-environment"},{"type":"module","title":"Examples - PlugLocale.Browser","doc":"When:\n\n  * `:default_locale` option is set to `\"en\"`\n  * `:locales` option is set to `[\"en\", \"zh\"]`\n\nFor users in an English-speaking environment:\n\n  * `https://example.com/en` will be responded directly.\n  * `https://example.com/` will be redirected to `https://example.com/en`.\n  * `https://example.com/path` will be redirected to `https://example.com/en/path`.\n  * `https://example.com/unknown` will be redirected to `https://example.com/en`.\n  * ...\n\nFor users in an Chinese-speaking environment:\n\n  * `https://example.com/zh` will be responded directly.\n  * `https://example.com/` will be redirected to `https://example.com/zh`.\n  * `https://example.com/path` will be redirected to `https://example.com/zh/path`.\n  * `https://example.com/unknown` will be redirected to `https://example.com/zh`.\n  * ...","ref":"PlugLocale.Browser.html#module-examples"},{"type":"function","title":"PlugLocale.Browser.build_locale_path/2","doc":"Builds a localized path for current path.\n\n> Note: the locale passed to this function won't be casted by the function\n> which is specified by `:cast_locale_by` option.","ref":"PlugLocale.Browser.html#build_locale_path/2"},{"type":"function","title":"Examples - PlugLocale.Browser.build_locale_path/2","doc":"# the request path of conn is /posts/7\n    iex> build_locale_path(conn, \"en\")\n    \"/en/posts/7\"\n\n    # the request path of conn is /en/posts/7\n    iex> build_locale_path(conn, \"zh\")\n    \"/zh/posts/7\"","ref":"PlugLocale.Browser.html#build_locale_path/2-examples"},{"type":"function","title":"PlugLocale.Browser.put_locale_resp_cookie/3","doc":"Puts a response cookie for locale in the connection.\n\nThis is a simple wrapper around `Plug.Conn.put_resp_cookie/4`. See its docs\nfor more details.","ref":"PlugLocale.Browser.html#put_locale_resp_cookie/3"},{"type":"function","title":"Examples - PlugLocale.Browser.put_locale_resp_cookie/3","doc":"iex> put_locale_resp_cookie(conn, \"en\")\n    iex> put_locale_resp_cookie(conn, \"zh\", max_age: 365 * 24 * 60 * 60)","ref":"PlugLocale.Browser.html#put_locale_resp_cookie/3-examples"},{"type":"function","title":"Use cases - PlugLocale.Browser.put_locale_resp_cookie/3","doc":"Use this function to persistent current locale into cookie, then subsequent\nrequests can directly read the locale from the cookie.\n\n    defmodule DemoWeb.PlugBrowserLocalization do\n      use Plug.Builder\n    \n      plug PlugLocale.Browser,\n        default_locale: \"en\",\n        locales: [\"en\", \"zh\"],\n        route_identifier: :locale,\n        assign_key: :locale\n    \n      plug :put_locale\n    \n      def put_locale(conn, _opts) do\n        if locale = conn.assigns[:locale] do\n          # integrate with gettext\n          Gettext.put_locale(locale)\n\n          # persistent current locale into cookie\n          PlugLocale.Browser.put_locale_resp_cookie(\n            conn,\n            locale,\n            max_age: 365 * 24 * 60 * 60\n          )\n        else\n          conn\n        end\n      end\n    end","ref":"PlugLocale.Browser.html#put_locale_resp_cookie/3-use-cases"},{"type":"module","title":"PlugLocale.Header","doc":"Puts locale into `assigns` storage according to given HTTP request header.\n\n> This is generally used by HTTP API for clients like mobile apps.\n\nBy default, the used HTTP request header is `x-client-locale`. Although\nthere's a standard HTTP header - `accept-language`, but it is better to\nuse a custom header, because it is simpler and has no historical baggage\nto overcome.","ref":"PlugLocale.Header.html"},{"type":"module","title":"Usage - PlugLocale.Header","doc":"First, we need to integrate this plug with other libraries, or this plug\nis useless. All you need is to construct a plug pipeline through\n`Plug.Builder`. For example:\n\n    defmodule DemoWeb.PlugMobileClientLocalization do\n      use Plug.Builder\n\n      plug PlugLocale.Header,\n        default_locale: \"en\",\n        locales: [\"en\", \"zh\"]\n\n      plug :put_locale\n\n      def put_locale(conn, _opts) do\n        if locale = conn.assigns[:locale] do\n          # integrate with gettext\n          Gettext.put_locale(locale)\n        end\n\n        conn\n      end\n    end\n\nThen, use it in router (following one is a Phoenix router, but `Plug.Router`\nis supported, too):\n\n    defmodule DemoWeb.Router do\n      use DemoWeb, :router\n\n      pipeline :api do\n        plug :accepts, [\"json\"]\n\n        # ...\n\n        plug DemoWeb.PlugMobileClientLocalization\n\n        # ...\n      end\n\n      scope \"/\", DemoWeb do\n        pipe_through :api\n\n        get \"/\", PageController, :index\n        # ...\n      end\n    end","ref":"PlugLocale.Header.html#module-usage"},{"type":"module","title":"Options - PlugLocale.Header","doc":"* `:default_locale` - the default locale.\n  * `:locales` - all the supported locales.\n    Default to `[]`.\n  * `:cast_locale_by` - the function for casting extracted or detected locales.\n    Default to `nil`.\n  * `:header_name` - the header for getting locale.\n    Default to `\"x-client-locale\"`.\n  * `:assign_key` - the key for putting value into `assigns` storage.\n    Default to `:locale`.\n\n#","ref":"PlugLocale.Header.html#module-options"},{"type":"module","title":"about `:cast_locale_by` option - PlugLocale.Header","doc":"By default, the value is `nil`, which means doing nothing. But, in practice,\nyou will need to use something meaningful.\n\nA possible implementation:\n\n    defmodule DemoWeb.I18n do\n      def cast_locale(locale) do\n        case locale do\n          # explicit matching on supported locales\n          locale when locale in [\"en\", \"zh\"] ->\n            locale\n\n          # fuzzy matching on en locale\n          \"en-\" <> _ ->\n            \"en\"\n\n          # fuzzy matching on zh locale\n          \"zh-\" <> _ ->\n            \"zh\"\n\n          # fallback for unsupported locales\n          _ ->\n            \"en\"\n        end\n      end\n    end\n\nThen, use above implementation for plug:\n\n    plug `PlugLocale.Header`,\n      default_locale: \"en\",\n      locales: [\"en\", \"zh\"],\n      cast_locale_by: &DemoWeb.I18n.cast_locale/1,\n      # ...","ref":"PlugLocale.Header.html#module-about-cast_locale_by-option"},{"type":"extras","title":"PlugLocale","doc":"# PlugLocale\n\n[![CI](https://github.com/cozy-elixir/plug_locale/actions/workflows/ci.yml/badge.svg)](https://github.com/cozy-elixir/plug_locale/actions/workflows/ci.yml)\n[![Hex.pm](https://img.shields.io/hexpm/v/plug_locale.svg)](https://hex.pm/packages/plug_locale)\n[![built with Nix](https://img.shields.io/badge/built%20with%20Nix-5277C3?logo=nixos&logoColor=white)](https://builtwithnix.org)\n\nPlugs for putting locale into `assigns` storage.","ref":"readme.html"},{"type":"extras","title":"Installation - PlugLocale","doc":"Add `:plug_locale` to the list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:plug_locale,  }\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - PlugLocale","doc":"Different from [`set_locale`](https://hex.pm/packages/set_locale), [`ex_cldr_plugs`](https://hex.pm/packages/ex_cldr_plugs), etc, `plug_locale`:\n\n- only does one simple thing - setting a locale-related assign (by default, it is `conn.assigns.locale`).\n- does not make any assumptions about the localization strategy, so it is not tightly bound to packages like [`gettext`](https://hex.pm/packages/gettext) or [`ex_cldr`](https://hex.pm/packages/ex_cldr).\n\nCheck out `PlugLocale.Browser`, `PlugLocale.Header` in [documentation](https://hexdocs.pm/plug_locale) for more details.","ref":"readme.html#usage"},{"type":"extras","title":"Thanks - PlugLocale","doc":"This library is built on the wisdom in following code:\n\n- [`set_locale`](https://hex.pm/packages/set_locale)","ref":"readme.html#thanks"},{"type":"extras","title":"License - PlugLocale","doc":"Apache License 2.0","ref":"readme.html#license"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.4.0 - Changelog","doc":"Breaking changes:\n\n- rename `:sanitize_locale_by` option to `:cast_locale_by`\n- rename `PlugLocale.WebBrowser.build_localized_path` to `PlugLocale.WebBrowser.build_locale_path`\n- rename `PlugLocale.WebBrowser` to `PlugLocale.Browser`\n\nNew Features:\n\n- add `PlugLocale.Browser.put_locale_resp_cookie/2` / `PlugLocale.Browser.put_locale_resp_cookie/3`","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"v0.3.0 - Changelog","doc":"Breaking changes:\n\n- rename `:sanitize_locale` option to `:sanitize_locale_by`","ref":"changelog.html#v0-3-0"}],"content_type":"text/markdown"}